#ifndef AO_H
#define AO_H

#ifndef AO_VERSION
#define AO_VERSION "20050512"
#endif

#ifndef AO_DEFAULT_MAX_FILE_SIZE
#define AO_DEFAULT_MAX_FILE_SIZE (1<<20)
#endif

/* These are potential duplicates. */
#ifndef EXTERN
#   ifdef __cplusplus
#	define EXTERN extern "C"
#   else
#	define EXTERN extern
#   endif
#endif
#ifndef ARGS
#   if defined(__STDC__) || defined(__cplusplus)
#	define ARGS(protos)    protos          /* ANSI C */
#   else /* !(__STDC__ || __cplusplus) */
#	define ARGS(protos)    ()              /* K&R C */
#   endif /* !(__STDC__ || __cplusplus) */
#endif
#ifndef NULLARGS
#   if defined(__STDC__) || defined(__cplusplus)
#       define NULLARGS    (void)
#   else
#       define NULLARGS    ()
#   endif
#endif
#ifndef const
#   if !defined(__STDC__) && !defined(__cplusplus)
#       define const
#   endif
#endif


#if defined(__GNUC__)
#   define UTIL_INLINE __inline__
#   if __GNUC__ > 2 || __GNUC_MINOR__ >= 7
#       define UTIL_UNUSED __attribute__ ((unused))
#   else
#       define UTIL_UNUSED
#   endif
#else
#   define UTIL_INLINE
#   define UTIL_UNUSED
#endif

#ifndef SIZEOF_VOID_P
#define SIZEOF_VOID_P 4
#endif
#ifndef SIZEOF_INT
#define SIZEOF_INT 4
#endif
#ifndef SIZEOF_LONG
#define SIZEOF_LONG 4
#endif

#if SIZEOF_VOID_P == 8 && SIZEOF_INT == 4
typedef long util_ptrint;
#else
typedef int util_ptrint;
#endif

#if defined(__STDC__) || defined(__cplusplus)
EXTERN void exit ARGS((int));
#endif

#define NIL(type)		((type *) 0)

/* No machines seem to have much of a problem with these */
#include <stdio.h>
#include <ctype.h>

/* some call it strings.h, some call it string.h; others, also have memory.h */
#if defined(__STDC__) || defined(__cplusplus) || defined(_IBMR2) || defined(ultrix)
#include <string.h>
#else
/* ANSI C string.h -- 1/11/88 Draft Standard */
extern char *strcpy(), *strncpy(), *strcat(), *strncat(), *strerror();
extern char *strpbrk(), *strtok(), *strchr(), *strrchr(), *strstr();
extern int strcoll(), strxfrm(), strncmp(), strlen(), strspn(), strcspn();
extern char *memmove(), *memccpy(), *memchr(), *memcpy(), *memset();
extern int memcmp(), strcmp();
#endif


#ifdef __STDC__
#include <assert.h>
#else
#ifndef NDEBUG
#define assert(ex) {\
    if (! (ex)) {\
	(void) fprintf(stderr,\
	    "Assertion failed: file %s, line %d\n\"%s\"\n",\
	    __FILE__, __LINE__, "ex");\
	(void) fflush(stdout);\
	abort();\
    }\
}
#else
#define assert(ex) ;
#endif
#endif


#define fail(why) {\
    (void) fprintf(stderr, "Fatal error: file %s, line %d\n%s\n",\
	__FILE__, __LINE__, why);\
    (void) fflush(stdout);\
    abort();\
}

#define ABS(a)			((a) < 0 ? -(a) : (a))
#define MAX(a,b)		((a) > (b) ? (a) : (b))
#define MIN(a,b)		((a) < (b) ? (a) : (b))
#define ALLOC(type, num)        \
    ((type *) malloc(sizeof(type) * (num)))
#define FREE(obj) free((void *) (obj))

/****************************************************************************
 *
 *
 *                AUXILIARY DEFINITIONS FOR AO
 *
 *
*****************************************************************************/

#ifndef __AOMSPY
#define malloc(s) \
    (__aoMspyMalloc(1, (s), __FILE__, __LINE__, 0, NULL))
#define free(p) (__aoMspyFree(p, __FILE__, __LINE__, 1))
#define realloc(p, s) (__aoMspyRealloc((p), (s), __FILE__, __LINE__))
#define calloc(n, s) \
    (__aoMspyMalloc((n), (s), __FILE__, __LINE__, 1, NULL))
#define strdup(s) \
    (__aoMspyStrdup((s), __FILE__, __LINE__))
#ifdef _GNU_SOURCE
#define strndup(s, n) \
    (__aoMspyStrndup((s), (n), __FILE__, __LINE__))
#endif
#else
#include <stdlib.h>
#endif

extern char  __aoAsRegisterIgnoreMask;
extern int   __aoAsBypassDiagnostics;
extern char  __aoAsModifiedRegisterFlags;
extern char *__aoAsRegisterNames[];
extern void *__aoAsStoredData;
extern void *__aoAsBeginDataSegment;
extern int   __aoAsStoredDataSize;

void   __aoAsWarnNumPush(unsigned char *, char *, int);
int    __aoAsGetNumPush(unsigned char *);
int    __aoAsFlagNoActivationBlock(unsigned char *, char *);
int    __aoAsHasActivationBlock(unsigned char *);
void   __aoAsWarnPusha(unsigned char *, char *);
int    __aoAsUsePusha(unsigned char *);
int    __aoAsFlagActivationBlockAnomaly(void *funcPtr,     // Function pointer
					void *paramPtr,    // Address of parameters
					int paramSz,       // Size of params in bytes
					void *resultPtr,   // Address to move result
					int resultSz,      // Size of result in bytes
					int toEAX,         // Register to move to eax
					char *funcName,    // Function name
					int *returned,     // Send back returned value
					int expectedDiag); // Expected diagnostics
int    __aoAsGetStackSize(void *);
int    __aoAsBypassCall(void *, // Function pointer
			void *, // Parameter area
			int,    // Size of params in bytes packets
			void *, // Result is moved to this address
			int,    // Size of result in bytes packets
			int);   // Register to transfer to eax
void   __aoAsStoreRegisters();
void   __aoAsCheckModifiedRegisters();
int    __aoAsFlagModifiedRegisters();
void   __aoAsCopyDataSegment(void *begin, void *end);
void   __aoAsRestoreDataSegment();

void   __aoMspySetMemLimit(int l);
int    __aoMspyGetMemLimit();
int    __aoMspyLeakByteCount();
int    __aoMspyLeakCount();
int    __aoMspyStrayFreeCount();
size_t __aoMspyUsedMemory();
size_t __aoMspyPeakMemory();
void   __aoMspyReport();
void   __aoMspySetInternalTrace(int t);
void  *__aoMspyMalloc(size_t nmemb, size_t size, char *file, int line, int clear, void *givenPtr);
char  *__aoMspyStrdup(const char *s, char *file, int line);
#ifdef _GNU_SOURCE
char  *__aoMspyStrndup(const char *s, size_t n, char *file, int line);
#endif
void   __aoMspyFree(void *ptr, char *file, int line, int perform);
void  *__aoMspyRealloc(void *ptr, size_t size, char *file, int line);

float  __aoMiscRan3(long *seed);
int    __aoMiscRanByte();
void   __aoMiscSetTimeOutAlarm(int seconds);
long   __aoMiscCpuTime(void);
char  *__aoMiscPrintTime(unsigned long);
void   __aoMiscPrintCpuStats(FILE *);

/* Functions to get error and warning prefixes */
const char *__aoGetWarningPrefix();
const char *__aoGetErrorPrefix();
const char *__aoGetMessage(int a);

/* Functions to set maximum file size allowed to be produced */
void __aoSetDefaultMaxFileSize();
void __aoSetMaxFileSize(int s);

// Constants to access different messages
#define __AO_MSG_WARNING                0
#define __AO_MSG_ERROR                  1
#define __AO_MSG_TIMEOUT                2
#define __AO_MSG_REGISTERS              3
#define __AO_MSG_REGISTER               4
#define __AO_MSG_CONJUNCTION            5
#define __AO_MSG_HAVE_BEEN_MODIFIED     6
#define __AO_MSG_HAS_BEEN_MODIFIED      7
#define __AO_MSG_REDUCE_REGISTERS       8
#define __AO_MSG_NO_REGISTERS           9
#define __AO_MSG_NO_ACTIVATION_BLOCK   10
#define __AO_MSG_USE_PUSHA             11
#define __AO_MSG_NO_PARAM_MODIFICATION 12
#define __AO_MSG_PARAM_MODIFICATION    13
#define __AO_MSG_DIFFERENT_RESULT      14
#define __AO_MSG_NO_RESULT_CHANGE      15
#define __AO_MSG_CHANGE_IN_STACK       16
#define __AO_MSG_MM_REPORT             17
#define __AO_MSG_MM_REPORT_UNDERLINE   18
#define __AO_MSG_MM_CORRECT_MNGMT      19
#define __AO_MSG_MM_INCORRECT_MNGMT    20
#define __AO_MSG_MM_CORRUPTED_MEMORY_1 21
#define __AO_MSG_MM_CORRUPTED_MEMORY_2 22
#define __AO_MSG_MM_CORRUPTED_MEMORY_3 23
#define __AO_MSG_MM_LEAK_EXPLAINED     24
#define __AO_MSG_MM_MEMORY_LEAK        25
#define __AO_MSG_MM_SIZE               26
#define __AO_MSG_MM_REPEATED           27
#define __AO_MSG_MM_LEAKS              28
#define __AO_MSG_MM_INCORRECT_FREES    29
#define __AO_MSG_MM_TOTAL_INC_FREES    30
#define __AO_MSG_MM_BEING_USED         31
#define __AO_MSG_MM_PEAK_MEMORY        32
#define __AO_MSG_MM_ATEXIT_FAILURE     33
#define __AO_MSG_TIMEOUT_REPROGRAM     34
#define __AO_MSG_STDOUT_TOO_LARGE      35
#define __AO_MSG_MM_ABOVELIMIT         36

#endif /* AO_H */
